/*
 *	XMTV
 *
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XMTV software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XMTV source distribution.
 *
 *	$Id: lamtv.c,v 1.3 2002/12/09 20:25:14 brbarret Exp $
 * 
 *	Function:	- lam_tv*() wrappers to lam_rtv*() calls
 *			- hide nodeid by searching for deamon
 *			- works for single static graphics daemon
 */

#include <stdlib.h>

#include <events.h>
#include <net.h>
#include <portable.h>
#include <preq.h>
#include <tvreq.h>
#include <typical.h>

/*
 * local variables
 */
static int4		mtvnode = NOTNODEID;	/* graphics daemon nodeid */

/*
 * local functions
 */
static int4		getmtvnode();

/*
 *	lam_tvclr
 *
 *	Function:	- nodeless interface to lam_rtvclr()
 */
int
lam_tvclr()

{
	int4		node;

	node = getmtvnode();

	if (node == NOTNODEID) {
		return(LAMERROR);
	} else {
		return(lam_rtvclr(node));
	}
}

/*
 *	lam_tvrect
 *
 *	Function:	- nodeless interface to lam_rtvrect()
 */
int
lam_tvrect(x, y, xlen, ylen, colours)

int4			x, y, xlen, ylen;
char			*colours;

{
	int4		node;

	node = getmtvnode();

	if (node == NOTNODEID) {
		return(LAMERROR);
	} else {
		return(lam_rtvrect(node, x, y, xlen, ylen, colours));
	}
}

/*
 *	lam_tvsnake
 *
 *	Function:	- nodeless interface to lam_rtvsnake()
 */
int
lam_tvsnake(x, len, colours)

int4			x, len;
char			*colours;

{
	int4		node;

	node = getmtvnode();

	if (node == NOTNODEID) {
		return(LAMERROR);
	} else {
		return(lam_rtvsnake(node, x, len, colours));
	}
}

/*
 *	lam_tvtext
 *
 *	Function:	- nodeless interface to lam_rtvtext()
 */
int
lam_tvtext(x, y, colour, text)

int4			x, y, colour;
char			*text;

{
	int4		node;

	node = getmtvnode();

	if (node == NOTNODEID) {
		return(LAMERROR);
	} else {
		return(lam_rtvtext(node, x, y, colour, text));
	}
}

/*
 *	lam_tvstate
 *
 *	Function:	- nodeless interface to lam_rtvstate()
 */
int
lam_tvstate(pstate)

struct tvstate		*pstate;

{
	int4		node;

	node = getmtvnode();

	if (node == NOTNODEID) {
		return(LAMERROR);
	} else {
		return(lam_rtvstate(node, pstate));
	}
}

/*
 *	getmtvnode
 *
 *	Function:	- get/cache graphics node if necessary
 *	Returns:	- graphics nodeid or NOTNODEID
 */
static int4
getmtvnode()

{
	struct pstate	*pps;		/* process stat array */
	int		i, j;
	int4		nnodes;		/* # nodes in multicomputer */
	int4		*pnodes;	/* nodes in multicomputer */
	int		nps;		/* ret'n # processes */

	if (mtvnode != NOTNODEID) {
		return(mtvnode);
	}

	nnodes = getnall();
	if (nnodes < 0) return(NOTNODEID);

	pnodes = (int4 *) malloc((unsigned) (nnodes * sizeof(int4)));
	if (pnodes == 0) return(NOTNODEID);

	if (getall(pnodes, nnodes)) {
		free((char *) pnodes);
		return(NOTNODEID);
	}
        
	pps = (struct pstate *) malloc((unsigned) PMAX *
			sizeof(struct pstate));

	if (pps == 0) {
		free((char *) pnodes);
		return(NOTNODEID);
	}

	for (i = 0; i < nnodes; ++i) {
		nps = rpstate(pnodes[i], SELECT_ALL, 0, pps, PMAX);
		if (nps < 0) return(NOTNODEID);

		for (j = 0; j < nps; ++j) {

			if (strcmp("xmtv", pps[j].ps_name) == 0) {
				mtvnode = pnodes[i];
				free((char *) pnodes);
				free((char *) pps);
				return(mtvnode);
			}
		}
	}

	free((char *) pnodes);
	free((char *) pps);
	return(NOTNODEID);
}
