/*
 *	Ohio Trollius
 *
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XMTV software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XMTV source distribution.
 *
 *	$Id: rtvrect.c,v 1.3 2002/12/09 20:25:14 brbarret Exp $
 * 
 *	Function:	- plot a rectangular region of pixels
 *	Accepts: 	- destination node
 *			- x ordinate of top-left corner
 *			- y ordinate of top-left corner
 *			- rectangle width (horizontal)
 *			- rectangle height (vertical)
 *			- array of pixel colours
 *	Returns:	- 0 or ERROR
 */

#include <events.h>
#include <net.h>
#include <portable.h>
#include <terror.h>
#include <typical.h>
#include <tvreq.h>

int
lam_rtvrect(node, x1, y1, xlen, ylen, colours)

int4			node;
int4			x1;
int4			y1;
int4			xlen;
int4			ylen;
char			*colours;

{
	struct nmsg	nhead;		/* message header */
	struct tvreq	*request;	/* ptr to mtvd request */
	int4		yrows;
/*
 * Check the trivial cases.
 */
	if ((xlen == 0) || (ylen == 0)) {
		return(0);
	}

	if ((xlen < 0) || (ylen < 0)) {
		errno = EINVAL;
		return(ERROR);
	}
/*
 * Set up the request message.
 */
 	request = (struct tvreq *) nhead.nh_data;
	request->tvq_req = TVQRECT;

	nhead.nh_node = node;
	nhead.nh_event = EVMTVD;
	nhead.nh_type = 0;
	nhead.nh_flags = 0;
/*
 * Packetize the rectangle by cutting it in slices along the y-axis.
 * The assumption is that no rectangle will have an xlen (width) larger
 * than MAXNMSGLEN, otherwise slicing along the x-axis would be needed too.
 */
	yrows = MAXNMSGLEN / xlen;

	request->tvq_x1 = x1;
	request->tvq_y1 = y1;
	request->tvq_xlen = xlen;
	nhead.nh_msg = colours;

	while (ylen > 0) {

		request->tvq_ylen = (yrows > ylen) ? ylen : yrows;
		nhead.nh_length = xlen * request->tvq_ylen;

		if (nsend(&nhead)) {
			return(ERROR);
		}

		request->tvq_y1 += request->tvq_ylen;
		nhead.nh_msg += nhead.nh_length;
		ylen -= request->tvq_ylen;
	}

	return(0);
}
