/*
 *	Ohio Trollius
 *
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XMTV software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XMTV source distribution.
 *
 *	$Id: rtvsnake.c,v 1.3 2002/12/09 20:25:14 brbarret Exp $
 * 
 *	Function:	- plot a snake-like region of pixels
 *	Accepts: 	- destination node
 *			- linear ordinate of first pixel
 *			- # pixels in the snake
 *			- array of pixel colours
 *	Returns:	- 0 or ERROR
 */

#include <events.h>
#include <net.h>
#include <portable.h>
#include <terror.h>
#include <typical.h>
#include <tvreq.h>

int
lam_rtvsnake(node, x1, len, colours)

int4			node;
int4			x1;
int4			len;
char			*colours;

{
	struct nmsg	nhead;		/* message header */
	struct tvreq	*request;	/* ptr to mtvd request */
	int4		chunk;
/*
 * Check the trivial cases.
 */
	if (len == 0) {
		return(0);
	}
	else if (len < 0) {
		errno = EINVAL;
		return(ERROR);
	}
/*
 * Set up the request message.
 */
 	request = (struct tvreq *) nhead.nh_data;
	request->tvq_req = TVQSNAKE;
	request->tvq_x1	 = x1;

	nhead.nh_node = node;
	nhead.nh_event = EVMTVD;
	nhead.nh_type = 0;
	nhead.nh_flags = 0;
	nhead.nh_msg = colours;
/*
 * Packetize the snake by chopping it into smaller 1-packet snakes.
 */
	while (len > 0) {

		chunk = (len > MAXNMSGLEN) ? MAXNMSGLEN : len;
		request->tvq_len = chunk;
		nhead.nh_length	= chunk;
		
		if (nsend(&nhead)) {
			return(ERROR);
		}

		request->tvq_x1 += chunk;
		nhead.nh_msg += chunk;
		len -= chunk;
	}

	return(0);
}
