/*
 *	XMTV
 *
 * Copyright (c) 2001-2002 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the XMTV software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * XMTV source distribution.
 *
 *	$Id: xmtv_ctl.c,v 1.3 2002/12/09 20:25:14 brbarret Exp $
 * 
 *	Function:	- control panel for main window
 */

#define _NO_PROTO

#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>

#include <stdio.h>
#include <string.h>

#include <portable.h>
#include <terror.h>
#include <xmtv.h>

/*
 * global functions
 */
Widget			xmtv_ctl_build();

/*
 * external functions
 */
extern void		xmtv_view_clear();

/*
 * local functions
 */
static void		ctl_ctl_cb();
static void		ctl_help_cb();

/*
 *	xmtv_ctl_build
 *
 *	Function:	- creates widgets for xmtv control panel
 *	Accepts:	- parent widget
 *	Returns:	- created widget
 */
Widget
xmtv_ctl_build(parent_w)

Widget			parent_w;

{
	Widget		menubar_w;
	Widget		mgr_w;		/* control area manager */
	Widget		w;
	XmString	str0, str1;

	mgr_w = XtVaCreateWidget("ctl_mgr",
		xmFormWidgetClass, parent_w,
		NULL);
/*
 * Create the menu bar.
 */
	str0 = XmStringCreateSimple("Control");
	str1 = XmStringCreateSimple("Help");
	menubar_w = XmVaCreateSimpleMenuBar(mgr_w, "ctl_bar",
		XmVaCASCADEBUTTON, str0, 'C',
		XmVaCASCADEBUTTON, str1, 'H',
		XmNtopAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNbottomAttachment, XmATTACH_FORM,
		NULL);
	XmStringFree(str0);
	XmStringFree(str1);

	if (w = XtNameToWidget(menubar_w, "button_1")) {
		XtVaSetValues(menubar_w, XmNmenuHelpWidget, w, NULL);
	}
/*
 * Create the pulldown menu for "Control".
 */
	str0 = XmStringCreateSimple("Clear");
	str1 = XmStringCreateSimple("Quit");
	w = XmVaCreateSimplePulldownMenu(menubar_w, "Ctl_menu", 0, ctl_ctl_cb,
		XmVaPUSHBUTTON, str0, 'C', NULL, NULL,
		XmVaSEPARATOR,
		XmVaPUSHBUTTON, str1, 'Q', NULL, NULL,
		NULL);
/*
 * XmVaSEPARATOR seems buggy.  It makes following buttons forget labels.
 */
	if (w = XtNameToWidget(w, "button_1")) {
		XtVaSetValues(w, XmNlabelString, str1, XmNmnemonic, 'Q', NULL);
	}

	XmStringFree(str0);
	XmStringFree(str1);
/*
 * Create the pulldown menu for "Help".
 */
	str0 = XmStringCreateSimple("Help");
	XmVaCreateSimplePulldownMenu(menubar_w, "Help_menu", 1, ctl_help_cb,
		XmVaPUSHBUTTON, str0, 'H', NULL, NULL,
		XmNsensitive, False,
		NULL);
	XmStringFree(str0);

	XtManageChild(menubar_w);

	XtManageChild(mgr_w);
	return(mgr_w);
}

/*
 *	ctl_ctl_cb
 *
 *	Function:	- invokes items from the Control menu
 *	Accepts:	- menu widget
 *			- selected menu item
 */
static void
ctl_ctl_cb(w, item)

Widget			w;
int			item;

{
	if (item == 0) {
		xmtv_view_clear();
	}
	else if (item == 1) {
		exit(0);
	}
}

/*
 *	ctl_help_cb
 *
 *	Function:	- just a stub for now
 */
static void
ctl_help_cb()

{
}
