/*
 *	XMTV
 *	Copyright 1996 The Ohio State University
 *	RBD
 *
 *	$Id: xmtv_view.c,v 1.3 2003/01/09 18:13:22 brbarret Exp $
 * 
 *	Function:	- overall application view in main window
 */

#define _NO_PROTO

#include <Xm/DrawingA.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/ScrolledW.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <events.h>
#include <net.h>
#include <portable.h>
#include <terror.h>
#include <typical.h>
#include <xmtv.h>

#include "lamlogo.xpm"

/*
 * global functions
 */
Widget			xmtv_view_build();
void			xmtv_view_clear();
void			xmtv_view_rect();
void			xmtv_view_snake();
void			xmtv_view_text();
void			xmtv_view_state();

/*
 * external functions
 */
extern void		xmtv_seticon();
extern void		xmtv_setminmax();
extern void		xpm_build();

/*
 * external variables
 */
extern AppData		app_res;		/* application resources */

/*
 * local functions
 */
static void		cb_draw();
static void		cmapcreate();
static void		drawclear();
static void		gccreate();
static void		getrgb();
static void		plotchar();
static void		setcolour();

/*
 * local variables
 */
static int		drawwidth;	/* drawing area width */
static int		drawheight;	/* drawing area heigth */
static int		ncolours;	/* # of graphics colours */
static int		fl_autoclr = 1;	/* auto-clear-screen flag */
static unsigned int	depth;		/* screen depth */
static Widget		draw_w;		/* drawing area widget */
static Display		*disp;		/* drawing area display */
static Drawable		root;		/* root window */
static GC		fggc;		/* foreground pen */
static GC		bggc;		/* background pen */
static Pixmap		pixmap;		/* copy of drawing area */
static Pixel		*colours;	/* graphics colour map */

/*
 *	xmtv_view_build
 *
 *	Function:	- build widgets for graphics viewing subwindow
 *	Accepts:	- parent widget
 *	Returns:	- created widget
 */
Widget
xmtv_view_build(parent_w)

Widget			parent_w;

{
	Widget		view_w;		/* subwindow widget */
	Widget		scroll_w;	/* scrolled window widget */

	view_w = XtVaCreateWidget("view_mgr",
		xmFormWidgetClass, parent_w,
		NULL);
/*
 * Build the scrolled window.
 */
	drawwidth = app_res.ap_xsize;
	drawheight = app_res.ap_ysize;

	scroll_w = XtVaCreateManagedWidget("view_scrwin",
			xmScrolledWindowWidgetClass, view_w,
			XmNwidth, drawwidth + XMTV_DRAWXEXTRA,
			XmNheight, drawheight + XMTV_DRAWYEXTRA,
			XmNscrollingPolicy, XmAUTOMATIC,
			XmNspacing, 0,
			XmNtopAttachment, XmATTACH_FORM,
			XmNtopOffset, 2,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_FORM,
			NULL);

	xmtv_setminmax(drawwidth / 2, drawheight / 2, drawwidth, drawheight);
/*
 * Build the drawing area.
 */
	draw_w = XtVaCreateManagedWidget("view_draw",
			xmDrawingAreaWidgetClass, scroll_w,
			XmNwidth, drawwidth,
			XmNheight, drawheight,
			NULL);
	XtAddCallback(draw_w, XmNexposeCallback, cb_draw, NULL);

	XtManageChild(draw_w);

	XtManageChild(scroll_w);

	XtManageChild(view_w);
/*
 * Create the graphic contexts and the colour map.
 */
	disp = XtDisplay(draw_w);
	root = RootWindowOfScreen(XtScreen(draw_w));
	depth = DefaultDepthOfScreen(XtScreen(draw_w));

	gccreate();
	cmapcreate();

	return(view_w);
}

/*
 *	xmtv_view_clear
 *
 *	Function:	- clear the view window
 */
void
xmtv_view_clear()

{
	drawclear();
	XCopyArea(disp, pixmap, XtWindow(draw_w), fggc,
			0, 0, drawwidth, drawheight, 0, 0);
	fl_autoclr = 0;
}

/*
 *	gccreate
 *
 *	Function:	- create graphic contexts
 */
static void
gccreate()

{
	Pixel		fg, bg;		/* colours of drawing area */
	XGCValues	gcval;		/* graphic context info */
	Pixmap		logopix;	/* logo pixmap */
	int		logowidth;	/* width of LAM logo */
	int		logoheight;	/* height of LAM logo */
/*
 * Create the graphic contexts (foreground and background).
 */
	XtVaGetValues(draw_w, XmNforeground, &fg, XmNbackground, &bg, NULL);

	gcval.foreground = fg;
	gcval.background = bg;

	fggc = XCreateGC(disp, root, (GCForeground | GCBackground), &gcval);

	gcval.foreground = bg;
	bggc = XCreateGC(disp, root, GCForeground, &gcval);
/*
 * Create the pixmap where the drawing area is saved.
 */
	pixmap = XCreatePixmap(disp, root, drawwidth, drawheight, depth);

	drawclear();
/*
 * Create the LAM logo and set it as the icon.
 */
	xpm_build(draw_w, lamlogo_xpm, (Pixel) 0, &logopix, &logowidth,
		&logoheight);
	
	if (logopix != XmUNSPECIFIED_PIXMAP) {
		xmtv_seticon(logopix);
/*
 * Copy the logo to the initial screen.
 */
		if ((logowidth <= drawwidth) &&
				(logoheight <= drawheight)) {
			XCopyArea(disp, logopix, pixmap, fggc,
				0, 0, logowidth, logoheight,
				(drawwidth - logowidth) / 2,
				(drawheight - logoheight) / 2);
		}
	}
}

/*
 *	cmapcreate
 *
 *	Function:	- create the proper colour map for the display
 */
static void
cmapcreate()

{
	Colormap	cmap;		/* colour map */
	Visual		*visual;	/* default visual */
	XColor		xcol;		/* colour structure */
	int		i;		/* favourite index */
	int		r, g, b;	/* RGB values */

	cmap = DefaultColormapOfScreen(XtScreen(draw_w));
	visual = DefaultVisualOfScreen(XtScreen(draw_w));
/*
 * Set the # of graphics colours to use.
 */
	printf("depth: %d\n", depth);
	switch (depth) {
	case 1:
	  ncolours = 2;
	  break;
	case 8:
	  ncolours = XMTV_NCOL8BIT;
	case 15:
	case 16:
	case 24:
	case 32:
	  ncolours = XMTV_NCOL24BIT;
	  break;
	default:
		fprintf(stderr, "xmtv: %d-bit screens not supported\n", depth);
		kexit(1);
	}
/*
 * Allocate the graphics colour map.
 */
	colours = (Pixel *) malloc((unsigned) ncolours * sizeof(Pixel));
	if (colours == 0) {
		terror("xmtv (malloc)");
		kexit(errno);
	}
/*
 * Allocate colours according to display type.
 */
	if (depth == 1) {
		colours[0] = BlackPixelOfScreen(XtScreen(draw_w));
		colours[0] = WhitePixelOfScreen(XtScreen(draw_w));
	} else {
		switch(visual->class) {

		case PseudoColor:
		case StaticColor:
		case DirectColor:
		case TrueColor:

		for (i = 0; i < ncolours; ++i) {

			getrgb(i, ncolours, XMTV_MAXRGB, &r, &g, &b);

			xcol.red = r; xcol.green = g; xcol.blue = b;

			if ( ! XAllocColor(disp, cmap, &xcol)) {
				fprintf(stderr,
				"xmtv: cannot alloc. colour %d (%d, %d, %d)\n",
						i, r, g, b);

				xcol.pixel = i;
			}

			colours[i] = xcol.pixel;
		}
		break;

		case GrayScale:
		case StaticGray:

		for (i = 0; i < ncolours; ++i) {

			xcol.green = xcol.blue = 0;
			xcol.red = (XMTV_MAXRGB * i) / (ncolours - 1);
			
			if ( ! XAllocColor(disp, cmap, &xcol)) {
				fprintf(stderr,
					"xmtv: cannot alloc. gray %d (%d)\n",
					i, xcol.red);

				xcol.pixel = colours[0];
			}

			colours[i] = xcol.pixel;
		}
		break;

		default:
		fprintf(stderr, "xmtv: unknown visual class\n");
		kexit(1);
		}
	}
}

/*
 *	cb_draw
 *
 *	Function:	- callback function for drawing area events
 *			- handles expose events
 *	Accepts:	- widget
 *			- client data
 *			- ptr callback struct
 */
static void
cb_draw(w, cdata, p_callback)

Widget			w;
XtPointer		cdata;
XmDrawingAreaCallbackStruct
			*p_callback;

{
	XEvent		*event;		/* activating event */

	event = p_callback->event;
/*
 * Handle the expose event.
 */
	if (p_callback->reason == XmCR_EXPOSE) {
		XCopyArea(event->xany.display, pixmap, event->xany.window,
				fggc, 0, 0, drawwidth, drawheight, 0, 0);
	}
}

/*
 *	drawclear
 *
 *	Function:	- clear the drawing area
 */
static void
drawclear()

{
	XFillRectangle(disp, pixmap, bggc, 0, 0, drawwidth, drawheight);
}

/*
 *	getrgb
 *
 *	Function:	- set RGB colour values in a "linear" spectrum
 *	Accepts:	- colour index
 *			- total # colours
 *			- max R/G/B value
 *			- ptr red (returned value)
 *			- ptr green (returned value)
 *			- ptr blue (returned value)
 */
static void
getrgb(idx, ncol, maxrgb, pred, pgreen, pblue)

int			idx;
int			ncol;
int			maxrgb;
int			*pred;
int			*pgreen;
int			*pblue;

{
	int		region;		/* region number */
	int		regloc;		/* location in region */
	int		fullloc;	/* full region location */
	double		factor;		/* colour scaling factor */
/*
 * Handle boundary cases.
 */
	if (idx == 0) {
		*pred = *pgreen = *pblue = 0;
		return;
	}

	if (idx >= (ncol - 1)) {
		*pred = *pgreen = *pblue = maxrgb;
		return;
	}
/*
 * Location colour region in spectrum.
 * The spectrum is divided into 12 regions in which RGB values change
 * linearly to give: black-red-yellow-green-light blue-blue-purple-white.
 */
	fullloc = ncol / 12;
	region = idx / fullloc;
	regloc = idx % fullloc;
	factor = ((double) maxrgb * 3) / ncol;

	switch(region) {
/*
 * Add red very fast (black -> red).
 */
	case 0:
		*pred = (7 * factor * regloc) + 0.5;
		if (*pred > maxrgb) *pred = maxrgb;
		*pgreen = *pblue = 0;
		break;
/*
 * Add green quickly (red -> yellow).
 */
	case 1:
		*pred = maxrgb;
		*pgreen = (3 * factor * regloc) + 0.5;
		*pblue = 0;
		break;
/*
 * Finish adding green slowly (red -> yellow).
 */
	case 2:
		*pred = maxrgb;
		*pgreen = maxrgb - (factor * (fullloc - regloc)) + 0.5;
		*pblue = 0;
		break;
/*
 * Remove red slowly (yellow -> green).
 */
	case 3:
		*pred = maxrgb - (factor * regloc) + 0.5;
		*pgreen = maxrgb;
		*pblue = 0;
		break;
/*
 * Finish removing red fast (yellow -> green).
 */
	case 4:
		*pred = (3 * factor * (fullloc - regloc)) + 0.5;
		*pgreen = maxrgb;
		*pblue = 0;
		break;
/*
 * Add blue fast (green -> light blue).
 */
	case 5:
		*pred = 0;
		*pgreen = maxrgb;
		*pblue = (3 * factor * regloc) + 0.5;
		break;
/*
 * Finish adding blue slowly (green -> light blue).
 */
	case 6:
		*pred = 0;
		*pgreen = maxrgb;
		*pblue = maxrgb - (factor * (fullloc - regloc)) + 0.5;
		break;
/*
 * Remove green slowly (light blue -> blue).
 */
	case 7:
		*pred = 0;
		*pgreen = maxrgb - (factor * regloc) + 0.5;
		*pblue = maxrgb;
		break;
/*
 * Finish removing green fast (light blue -> blue).
 */
	case 8:
		*pred = 0;
		*pgreen = (3 * factor * (fullloc - regloc)) + 0.5;
		*pblue = maxrgb;
		break;
/*
 * Add red fast (blue -> purple).
 */
	case 9:
		*pred = (3 * factor * regloc) + 0.5;
		*pgreen = 0;
		*pblue = maxrgb;
		break;
/*
 * Finish adding red slowly (blue -> purple).
 */
	case 10:
		*pred = maxrgb - (factor * (fullloc - regloc)) + 0.5;
		*pgreen = 0;
		*pblue = maxrgb;
		break;
/*
 * Add green very fast (purple -> white).
 */
	case 11:
	default:
		*pred = *pblue = maxrgb;
		*pgreen = (4 * factor * regloc) + 0.5;
		break;
	}
}

/*
 *	xmtv_view_rect
 *
 *	Function:	- plot a multi-coloured rectangle
 *			- handles boundary clipping
 *	Accepts:	- x ordinate of top-left corner
 *			- y ordinate of top-left corner
 *			- width
 *			- height
 *			- array of pixel colours
 */
void
xmtv_view_rect(x, y, width, height, pixcols)

int4			x;
int4			y;
int4			width;
int4			height;
char			*pixcols;

{
	int		xfirst;		/* first visible x ordinate */
	int		xlast;		/* last visible x ordinate */
	int		yfirst;		/* first visible y ordinate */
	int		ylast;		/* last visible y ordinate */
	int		i, j;		/* favourite indices */
	int		fl_mod;		/* pixmap modified flag */
	char		*pi, *pj;	/* favourite pointers */

	if (fl_autoclr) xmtv_view_clear();

	if ((width <= 0) || (height <= 0)) {
		return;
	}
/*
 * Determine the visible rectangle.
 */
	xfirst = (x < 0) ? 0 : x;
	yfirst = (y < 0) ? 0 : y;

	xlast = x + width - 1;
	ylast = y + height - 1;

	if (xlast >= drawwidth) xlast = drawwidth - 1;
	if (ylast >= drawheight) ylast = drawheight - 1;
/*
 * Plot the rectangle.
 */
	fl_mod = 0;

	pj = pixcols + (xfirst - x) + ((yfirst - y) * width);

	for (j = yfirst; j <= ylast; ++j, pj += width) {
		for (i = xfirst, pi = pj; i <= xlast; ++i, ++pi) {
			fl_mod = 1;
			setcolour((int) *pi);
			XDrawPoint(disp, pixmap, fggc, i, j);
		}
	}

	if (fl_mod) {
		XCopyArea(disp, pixmap, XtWindow(draw_w), fggc,
				xfirst, yfirst, xlast - xfirst + 1,
				ylast - yfirst + 1, xfirst, yfirst);
	}
}

/*
 *	xmtv_view_snake
 *
 *	Function:	- plot multi-coloured snake-like region
 *			- handles boundary clipping
 *	Accepts:	- x snake-ordinate
 *			- length
 *			- array of pixel colours
 */
void
xmtv_view_snake(x, length, pixcols)

int4			x;
int4			length;
char			*pixcols;

{
	int		xfirst;		/* first visible x ordinate */
	int		xlast;		/* last visible x ordinate */
	int		xp, yp;		/* point coordinates */
	int		xmin, xmax;	/* min/max point x ordinates */
	int		ymin;		/* min. point y ordinate */
	int		i;		/* favourite index */
	int		fl_mod;		/* pixmap modified flag */
	char		*p;		/* favourite pointer */

	if (fl_autoclr) xmtv_view_clear();

	if (length <= 0) {
		return;
	}
/*
 * Determine the visible snake.
 */
	xfirst = (x < 0) ? 0 : x;

	xlast = x + length - 1;
	if (xlast >= (drawwidth * drawheight))
			xlast = (drawwidth * drawheight) - 1;
/*
 * Plot the snake.
 */
	xp = xmin = xmax = xfirst % drawwidth;
	yp = ymin = xfirst / drawwidth;
	fl_mod = 0;

	p = pixcols + xfirst - x;

	for (i = xfirst; i <= xlast; ++i, ++p) {
		setcolour((int) *p);
		XDrawPoint(disp, pixmap, fggc, xp, yp);
		fl_mod = 1;

		if (++xp >= drawwidth) {
			xmax = drawwidth;
			xp = xmin = 0;
			++yp;
		}
	}

	if (xp > xmax) xmax = xp;

	if (fl_mod) {
		XCopyArea(disp, pixmap, XtWindow(draw_w), fggc,
			xmin, ymin, xmax - xmin, yp - ymin + 1, xmin, ymin);
	}
}

/*
 *	xmtv_view_text
 *
 *	Function:	- plot a text string
 *			- handles boundary clipping
 *	Accepts:	- x text ordinate
 *			- y text ordinate
 *			- colour
 *			- text string
 */
void
xmtv_view_text(x, y, col, text)

int4			x;
int4			y;
int4			col;
char			*text;

{
	int		xfirst;		/* first visible x ordinate */
	int		xlast;		/* last visible x ordinate */
	int		length;		/* string length */
	int		i;		/* favourite index */
	int		fl_mod;		/* pixmap modified flag */
	char		*p;		/* favourite pointers */

	if (fl_autoclr) xmtv_view_clear();

	length = strlen(text);
	if ((length <= 0) || (y < 0) || (y >= (drawheight / XMTV_YCHAR))) {
		return;
	}
/*
 * Determine the visible string.
 */
	xfirst = (x < 0) ? 0 : x;
	xlast = x + length - 1;
	if (xlast >= (drawwidth / XMTV_XCHAR))
			xlast = (drawwidth / XMTV_XCHAR) - 1;
/*
 * Plot the text string.
 */
	fl_mod = 0;
	setcolour((int) col);

	p = text + (xfirst - x);

	for (i = xfirst; i <= xlast; ++i, ++p) {
		plotchar(i, y, (int) *p);
		fl_mod = 1;
	}

	if (fl_mod) {
		XCopyArea(disp, pixmap, XtWindow(draw_w), fggc,
			xfirst * XMTV_XCHAR, y * XMTV_YCHAR,
			(xlast - xfirst + 1) * XMTV_XCHAR, XMTV_YCHAR,
			xfirst * XMTV_XCHAR, y * XMTV_YCHAR);
	}
}

/*
 *	xmtv_view_state
 *
 *	Function:	- get graphics state information
 *	Accepts:	- ptr pixel width (returned value)
 *			- ptr pixel height (returned value)
 *			- ptr # colours (returned value)
 *			- ptr character width (returned value)
 *			- ptr character height (returned value)
 */
void
xmtv_view_state(ppixw, ppixh, pncol, pchrw, pchrh)

int4			*ppixw;
int4			*ppixh;
int4			*pncol;
int4			*pchrw;
int4			*pchrh;

{
	*ppixw = drawwidth;
	*ppixh = drawheight;
	*pncol = ncolours;
	*pchrw = drawwidth / XMTV_XCHAR;
	*pchrh = drawheight / XMTV_YCHAR;
}

/*
 *	setcolour
 *
 *	Function:	- set the pen colour
 *	Accepts:	- colour map index
 */
static void
setcolour(colidx)

int			colidx;

{
	XSetForeground(disp, fggc, colours[colidx]);
}

/*
 *	plotchar
 *
 *	Function:	- plot a character on the screen
 *	Accepts:	- x character ordinate
 *			- y character ordinate
 *			- character
 */
static void
plotchar(x, y, chr)

int			x;
int			y;
int			chr;

{
	int		i, j;		/* favourite indices */
	int		row;		/* row pixels */
	int		mask;		/* row pixel mask */
	unsigned char	*p;		/* ptr character map */

	if ((chr < 0) || (chr >= XMTV_NCHAR)) {
		return;
	}
/*
 * Plot the character pixels row by row.
 */
	x *= XMTV_XCHAR;
	y *= XMTV_YCHAR;

	p = charset[chr].cm_rows;

	for (i = 0; i < XMTV_YCHAR; ++i, ++p, ++y) {
		row = *p;
		mask = 1 << (XMTV_XCHAR - 1);

		for (j = 0; j < XMTV_XCHAR; ++j, mask >>= 1) {
			if (row & mask) {
				XDrawPoint(disp, pixmap, fggc, x + j, y);
			}
		}
	}
}
